@@ -3,9 +3,11 @@
 import os
 import warnings
 import traceback
+from typing import Callable, Optional
 
 
 __all__ = [
+    "ChunkError",
     "ClosedFileError",
     "ClosedNodeError",
     "DataTypeWarning",
@@ -16,8 +18,11 @@ __all__ = [
     "FlavorWarning",
     "HDF5ExtError",
     "NaturalNameWarning",
+    "NoSuchChunkError",
     "NoSuchNodeError",
     "NodeError",
+    "NotChunkedError",
+    "NotChunkAlignedError",
     "OldIndexWarning",
     "PerformanceWarning",
     "UnclosedFileWarning",
@@ -67,8 +72,8 @@ class HDF5ExtError(RuntimeError):
 
     # NOTE: in order to avoid circular dependencies between modules the
     #       _dump_h5_backtrace method is set at initialization time in
-    #       the utilsExtenion.
-    _dump_h5_backtrace = None
+    #       the utilsextension.pyx.
+    _dump_h5_backtrace: Optional[Callable[[], list[tuple[str, int, str, str]]]] = None
 
     DEFAULT_H5_BACKTRACE_POLICY = "VERBOSE"
     """Default policy for HDF5 backtrace handling
@@ -94,7 +99,7 @@ class HDF5ExtError(RuntimeError):
     """
 
     @classmethod
-    def set_policy_from_env(cls):
+    def set_policy_from_env(cls) -> str:
         envmap = {
             "IGNORE": False,
             "FALSE": False,
@@ -117,7 +122,7 @@ class HDF5ExtError(RuntimeError):
 
         return oldvalue
 
-    def __init__(self, *args, **kargs):
+    def __init__(self, *args, **kargs) -> None:
 
         super().__init__(*args)
 
@@ -151,7 +156,7 @@ class HDF5ExtError(RuntimeError):
         else:
             self.h5backtrace = None
 
-    def __str__(self):
+    def __str__(self) -> str:
         """Returns a sting representation of the exception.
 
         The actual result depends on policy set in the initializer
@@ -182,7 +187,7 @@ class HDF5ExtError(RuntimeError):
 
         return msg
 
-    def format_h5_backtrace(self, backtrace=None):
+    def format_h5_backtrace(self, backtrace: Optional[list[tuple[str, int, str, str]]]=None) -> str:
         """Convert the HDF5 trace back represented as a list of tuples.
         (see :attr:`HDF5ExtError.h5backtrace`) into a string.
 
@@ -248,9 +253,9 @@ class NodeError(AttributeError, LookupError):
     with children, and other similarly invalid operations.
 
     A node in a PyTables database cannot be simply overwritten by
-    replacing it.  Instead, the old node must be removed explicitely
+    replacing it.  Instead, the old node must be removed explicitly
     before another one can take its place.  This is done to protect
-    interactive users from inadvertedly deleting whole trees of data by
+    interactive users from inadvertently deleting whole trees of data by
     a single erroneous command.
 
     """
@@ -359,7 +364,7 @@ class OldIndexWarning(Warning):
 
     This warning is issued when an index in an unsupported format is
     found.  The index will be marked as invalid and will behave as if
-    doesn't exist.
+    it doesn't exist.
 
     """
 
@@ -394,3 +399,47 @@ class UnclosedFileWarning(Warning):
     this warning.
     """
     pass
+
+
+class ChunkError(Exception):
+    """An operation related to direct chunk access failed.
+
+    This exception may be related with the properties of the dataset or the
+    chunk being accessed, or with how the chunk is being accessed.  It is a
+    base for more specific exceptions.
+
+    """
+    pass
+
+
+class NotChunkedError(ChunkError):
+    """A direct chunking operation was attempted on a non-chunked dataset.
+
+    For instance, chunk information was requested for a plain ``Array``
+    instance.
+
+    """
+    pass
+
+
+class NotChunkAlignedError(ChunkError):
+    """A direct chunk read/write operation was given coordinates that do not
+    match the chunk's start.
+
+    These operations require coordinates that are integer multiples of the
+    dataset's chunksize.
+
+    """
+    pass
+
+
+class NoSuchChunkError(ChunkError):
+    """The chunk with the given coordinates does not exist in storage.
+
+    The coordinates are within the dataset's shape, though.
+
+    This is only an error when the chunk is to be read.  Such a missing chunk
+    can be written, in which case it is created in storage.
+
+    """
+    pass
