@@ -1,42 +1,76 @@
-# -*- coding: utf-8 -*-
-
-########################################################################
-#
-# License: BSD
-# Created: October 14, 2002
-# Author: Francesc Alted - faltet@pytables.com
-#
-# $Id$
-#
-########################################################################
-
 """Here is defined the Leaf class."""
-
 import warnings
 import math
+import json
+from pathlib import Path
+from functools import lru_cache
+from typing import Any, Literal, NamedTuple, Optional, Union, TYPE_CHECKING
 
-import numpy
+import numpy as np
 
-from .flavor import (check_flavor, internal_flavor,
-                           alias_map as flavor_alias_map)
+from .flavor import (check_flavor, internal_flavor, toarray,
+                     alias_map as flavor_alias_map)
 from .node import Node
 from .filters import Filters
 from .utils import byteorders, lazyattr, SizeType
-from .exceptions import PerformanceWarning
-from . import utilsextension
+from .exceptions import (ChunkError, NoSuchChunkError,
+                         NotChunkAlignedError, NotChunkedError,
+                         PerformanceWarning)
+
+if TYPE_CHECKING:
+    from .group import Group
 
 
-def csformula(expected_mb):
+# These should be declared as type aliases,
+# but ``TypeAlias`` requires Python >= 3.10
+# and ``type`` statements require Python >= 3.12.
+
+# ``np.typing.NDArray[np.uint8]`` requires NumPy >= 1.21.
+NPByteArray = np.ndarray[tuple[int], np.dtype[np.uint8]]
+
+# ``Buffer`` requires Python >= 3.12.
+BufferLike = Union[bytes, bytearray, memoryview, NPByteArray]
+
+
+def read_cached_cpu_info() -> dict[str, Any]:
+    try:
+        with open(Path.home() / '.pytables-cpuinfo.json', 'r') as f:
+            return json.load(f)
+    except FileNotFoundError:
+        return {}
+
+
+def write_cached_cpu_info(cpu_info_dict: dict[str, Any]) -> None:
+    with open(Path.home() / '.pytables-cpuinfo.json', 'w') as f:
+        json.dump(cpu_info_dict, f, indent=4)
+
+
+@lru_cache(maxsize=1)
+def get_cpu_info() -> dict[str, Any]:
+    cached_info = read_cached_cpu_info()
+    if cached_info:
+        return cached_info
+
+    try:
+        import cpuinfo
+        cpu_info_dict = cpuinfo.get_cpu_info()
+        write_cached_cpu_info(cpu_info_dict)
+        return cpu_info_dict
+    except ImportError:
+        return {}
+
+
+def csformula(expected_mb: int) -> int:
     """Return the fitted chunksize for expected_mb."""
 
     # For a basesize of 8 KB, this will return:
     # 8 KB for datasets <= 1 MB
     # 1 MB for datasets >= 10 TB
-    basesize = 8 * 1024   # 8 KB is a good minimum
+    basesize = 8 * 1024  # 8 KB is a good minimum
     return basesize * int(2**math.log10(expected_mb))
 
 
-def limit_es(expected_mb):
+def limit_es(expected_mb: int) -> int:
     """Protection against creating too small or too large chunks."""
 
     if expected_mb < 1:        # < 1 MB
@@ -46,7 +80,7 @@ def limit_es(expected_mb):
     return expected_mb
 
 
-def calc_chunksize(expected_mb):
+def calc_chunksize(expected_mb: int) -> int:
     """Compute the optimum HDF5 chunksize for I/O purposes.
 
     Rational: HDF5 takes the data in bunches of chunksize length to
@@ -71,6 +105,46 @@ def calc_chunksize(expected_mb):
     return chunksize * 8
 
 
+class ChunkInfo(NamedTuple):
+    """Information about storage for a given chunk.
+
+    It may also refer to a chunk which is within the dataset's shape but that
+    does not exist in storage, i.e. a missing chunk.
+
+    An instance of this named tuple class contains the following information,
+    in field order:
+
+    .. attribute:: start
+
+        The coordinates in dataset items where the chunk starts, a tuple of
+        integers with the same rank as the dataset.  These coordinates are
+        always aligned with chunk boundaries.  Also present for missing
+        chunks.
+
+    .. attribute:: filter_mask
+
+        An integer where each active bit signals that the filter in its
+        position in the pipeline was disabled when storing the chunk.  For
+        instance, ``0b10`` disables shuffling, ``0b100`` disables szip, and so
+        on.  ``None`` for missing chunks.
+
+    .. attribute:: offset
+
+        An integer which indicates the offset in bytes of chunk data as it
+        exists in storage.  ``None`` for missing chunks.
+
+    .. attribute:: size
+
+        An integer which indicates the size in bytes of chunk data as it
+        exists in storage.  ``None`` for missing chunks.
+
+    """
+    start: Union[tuple[int, ...], None]
+    filter_mask: Union[int, None]
+    offset: Union[int, None]
+    size: Union[int, None]
+
+
 class Leaf(Node):
     """Abstract base class for all PyTables leaves.
 
@@ -122,11 +196,6 @@ class Leaf(Node):
 
     """
 
-    # Properties
-    # ~~~~~~~~~~
-
-    # Node property aliases
-    # `````````````````````
     # These are a little hard to override, but so are properties.
     attrs = Node._v_attrs
     """The associated AttributeSet instance - see :ref:`AttributeSetClassDescr`
@@ -135,15 +204,14 @@ class Leaf(Node):
     """A description for this node
     (This is an easier-to-write alias of :attr:`Node._v_title`)."""
 
-    # Read-only node property aliases
-    # ```````````````````````````````
     @property
-    def name(self):
-        """The name of this node in its parent group (This is an easier-to-write alias of :attr:`Node._v_name`)."""
+    def name(self) -> str:
+        """The name of this node in its parent group (This is an
+        easier-to-write alias of :attr:`Node._v_name`)."""
         return self._v_name
 
     @property
-    def chunkshape(self):
+    def chunkshape(self) -> tuple[int, ...]:
         """The HDF5 chunk size for chunked leaves (a tuple).
 
         This is read-only because you cannot change the chunk size of a
@@ -152,7 +220,7 @@ class Leaf(Node):
         return getattr(self, '_v_chunkshape', None)
 
     @property
-    def object_id(self):
+    def object_id(self) -> int:
         """A node identifier, which may change from run to run.
         (This is an easier-to-write alias of :attr:`Node._v_objectid`).
 
@@ -163,16 +231,14 @@ class Leaf(Node):
         return self._v_objectid
 
     @property
-    def ndim(self):
+    def ndim(self) -> int:
         """The number of dimensions of the leaf data.
 
         .. versionadded: 2.4"""
         return len(self.shape)
 
-    # Lazy read-only attributes
-    # `````````````````````````
     @lazyattr
-    def filters(self):
+    def filters(self) -> Filters:
         """Filter properties for this leaf.
 
         See Also
@@ -184,7 +250,7 @@ class Leaf(Node):
         return Filters._from_leaf(self)
 
     @property
-    def track_times(self):
+    def track_times(self) -> bool:
         """Whether timestamps for the leaf are recorded
 
         If the leaf is not a dataset, this will fail with HDF5ExtError.
@@ -197,11 +263,8 @@ class Leaf(Node):
         """
         return self._get_obj_track_times()
 
-    # Other properties
-    # ````````````````
-
     @property
-    def maindim(self):
+    def maindim(self) -> int:
         """The dimension along which iterators work.
 
         Its value is 0 (i.e. the first dimension) when the dataset is not
@@ -213,31 +276,31 @@ class Leaf(Node):
         return self.extdim
 
     @property
-    def flavor(self):
+    def flavor(self) -> Literal["numpy", "python"]:
         """The type of data object read from this leaf.
 
         It can be any of 'numpy' or 'python'.
 
         You can (and are encouraged to) use this property to get, set
         and delete the FLAVOR HDF5 attribute of the leaf. When the leaf
-        has no such attribute, the default flavor is used..
+        has no such attribute, the default flavor is used.
         """
 
         return self._flavor
 
     @flavor.setter
-    def flavor(self, flavor):
+    def flavor(self, flavor: Literal["numpy", "python"]) -> None:
         self._v_file._check_writable()
         check_flavor(flavor)
         self._v_attrs.FLAVOR = self._flavor = flavor  # logs the change
 
     @flavor.deleter
-    def flavor(self):
+    def flavor(self) -> None:
         del self._v_attrs.FLAVOR
         self._flavor = internal_flavor
 
     @property
-    def size_on_disk(self):
+    def size_on_disk(self) -> int:
         """
         The size of this leaf's data in bytes as it is stored on disk.  If the
         data is compressed, this shows the compressed size.  In the case of
@@ -246,22 +309,24 @@ class Leaf(Node):
         """
         return self._get_storage_size()
 
-    # Special methods
-    # ~~~~~~~~~~~~~~~
-    def __init__(self, parentnode, name,
-                 new=False, filters=None,
-                 byteorder=None, _log=True,
-                 track_times=True):
+    def __init__(self,
+                 parentnode: "Group",
+                 name: str,
+                 new: bool=False,
+                 filters: Optional[Filters]=None,
+                 byteorder: Literal["little", "big", None]=None,
+                 _log: bool=True,
+                 track_times: bool=True) -> None:
         self._v_new = new
         """Is this the first time the node has been created?"""
-        self.nrowsinbuf = None
+        self.nrowsinbuf: Optional[int] = None
         """
         The number of rows that fits in internal input buffers.
 
         You can change this to fine-tune the speed or memory
         requirements of your application.
         """
-        self._flavor = None
+        self._flavor: Literal["numpy", "python", None] = None
         """Private storage for the `flavor` property."""
 
         if new:
@@ -282,10 +347,9 @@ class Leaf(Node):
         # Existing filters need not be read since `filters`
         # is a lazy property that automatically handles their loading.
 
+        super().__init__(parentnode, name, _log)
 
-        super(Leaf, self).__init__(parentnode, name, _log)
-
-    def __len__(self):
+    def __len__(self) -> int:
         """Return the length of the main dimension of the leaf data.
 
         Please note that this may raise an OverflowError on 32-bit platforms
@@ -296,38 +360,30 @@ class Leaf(Node):
 
         return self.nrows
 
-    def __str__(self):
+    def __str__(self) -> str:
         """The string representation for this object is its pathname in the
         HDF5 object tree plus some additional metainfo."""
 
-        # Get this class name
-        classname = self.__class__.__name__
-        # The title
-        title = self._v_title
-        # The filters
-        filters = ""
+        filters = []
         if self.filters.fletcher32:
-            filters += ", fletcher32"
+            filters.append("fletcher32")
         if self.filters.complevel:
             if self.filters.shuffle:
-                filters += ", shuffle"
+                filters.append("shuffle")
             if self.filters.bitshuffle:
-                filters += ", bitshuffle"
-            filters += ", %s(%s)" % (self.filters.complib,
-                                     self.filters.complevel)
-        return "%s (%s%s%s) %r" % \
-               (self._v_pathname, classname, self.shape, filters, title)
+                filters.append("bitshuffle")
+            filters.append(f"{self.filters.complib}({self.filters.complevel})")
+        return (f"{self._v_pathname} ({self.__class__.__name__}"
+                f"{self.shape}{', '.join(filters)}) {self._v_title!r}")
 
-    # Private methods
-    # ~~~~~~~~~~~~~~~
-    def _g_post_init_hook(self):
+    def _g_post_init_hook(self) -> None:
         """Code to be run after node creation and before creation logging.
 
         This method gets or sets the flavor of the leaf.
 
         """
 
-        super(Leaf, self)._g_post_init_hook()
+        super()._g_post_init_hook()
         if self._v_new:  # set flavor of new node
             if self._flavor is None:
                 self._flavor = internal_flavor
@@ -341,7 +397,7 @@ class Leaf(Node):
             else:
                 self._flavor = internal_flavor
 
-    def _calc_chunkshape(self, expectedrows, rowsize, itemsize):
+    def _calc_chunkshape(self, expectedrows: int, rowsize: int, itemsize: int) -> tuple[int, ...]:
         """Calculate the shape for the HDF5 chunk."""
 
         # In case of a scalar shape, return the unit chunksize
@@ -352,6 +408,35 @@ class Leaf(Node):
         MB = 1024 * 1024
         expected_mb = (expectedrows * rowsize) // MB
         chunksize = calc_chunksize(expected_mb)
+        complib = self.filters.complib
+        if (complib is not None and
+            complib.startswith("blosc2") and
+            self._c_classid in ('TABLE', 'CARRAY', 'EARRAY')):
+            # Blosc2 can introspect into blocks, so we can increase the
+            # chunksize for improving HDF5 perf for its internal btree.
+            # For the time being, this has been implemented efficiently
+            # just for tables, but in the future *Array objects could also
+            # be included.
+            # Use a decent default value for chunksize
+            chunksize *= 16
+            # Now, go explore the L3 size and try to find a smarter chunksize
+            cpu_info = get_cpu_info()
+            if 'l3_cache_size' in cpu_info:
+                # In general, is a good idea to set the chunksize equal to L3
+                l3_cache_size = cpu_info['l3_cache_size']
+                # cpuinfo sometimes returns cache sizes as strings (like,
+                # "4096 KB"), so refuse the temptation to guess and use the
+                # value only when it is an actual int.
+                # Also, sometimes cpuinfo does not return a correct L3 size;
+                # so in general, enforcing L3 > L2 is a good sanity check.
+                l2_cache_size = cpu_info.get('l2_cache_size', "Not found")
+                if (type(l3_cache_size) is int and
+                    type(l2_cache_size) is int and
+                    l3_cache_size > l2_cache_size):
+                    chunksize = l3_cache_size
+            # In Blosc2, the chunksize cannot be larger than 2 GB - BLOSC2_MAX_BUFFERSIZE
+            if chunksize > 2**31 - 32:
+                chunksize = 2**31 - 32
 
         maindim = self.maindim
         # Compute the chunknitems
@@ -362,7 +447,7 @@ class Leaf(Node):
         chunkshape = list(self.shape)
         # Check whether trimming the main dimension is enough
         chunkshape[maindim] = 1
-        newchunknitems = numpy.prod(chunkshape, dtype=SizeType)
+        newchunknitems = np.prod(chunkshape, dtype=SizeType)
         if newchunknitems <= chunknitems:
             chunkshape[maindim] = chunknitems // newchunknitems
         else:
@@ -370,7 +455,7 @@ class Leaf(Node):
             for j in range(len(chunkshape)):
                 # Check whether trimming this dimension is enough
                 chunkshape[j] = 1
-                newchunknitems = numpy.prod(chunkshape, dtype=SizeType)
+                newchunknitems = np.prod(chunkshape, dtype=SizeType)
                 if newchunknitems <= chunknitems:
                     chunkshape[j] = chunknitems // newchunknitems
                     break
@@ -381,7 +466,7 @@ class Leaf(Node):
 
         return tuple(SizeType(s) for s in chunkshape)
 
-    def _calc_nrowsinbuf(self):
+    def _calc_nrowsinbuf(self) -> int:
         """Calculate the number of rows that fits on a PyTables buffer."""
 
         params = self._v_file.params
@@ -411,7 +496,12 @@ very small/large chunksize, you may want to increase/decrease it."""
         return nrowsinbuf
 
     # This method is appropriate for calls to __getitem__ methods
-    def _process_range(self, start, stop, step, dim=None, warn_negstep=True):
+    def _process_range(self,
+                       start: int,
+                       stop: int,
+                       step: int,
+                       dim: Optional[int]=None,
+                       warn_negstep: bool=True) -> tuple[int, int, int]:
         if dim is None:
             nrows = self.nrows  # self.shape[self.maindim]
         else:
@@ -420,19 +510,23 @@ very small/large chunksize, you may want to increase/decrease it."""
         if warn_negstep and step and step < 0:
             raise ValueError("slice step cannot be negative")
 
-        #if start is not None: start = long(start)
-        #if stop is not None: stop = long(stop)
-        #if step is not None: step = long(step)
+        # if start is not None: start = long(start)
+        # if stop is not None: stop = long(stop)
+        # if step is not None: step = long(step)
 
         return slice(start, stop, step).indices(int(nrows))
 
     # This method is appropriate for calls to read() methods
-    def _process_range_read(self, start, stop, step, warn_negstep=True):
+    def _process_range_read(self,
+                            start: int,
+                            stop: int,
+                            step: int,
+                            warn_negstep: bool=True) -> tuple[int, int, int]:
         nrows = self.nrows
         if start is not None and stop is None and step is None:
             # Protection against start greater than available records
             # nrows == 0 is a special case for empty objects
-            if nrows > 0 and start >= nrows:
+            if 0 < nrows <= start:
                 raise IndexError("start of range (%s) is greater than "
                                  "number of rows (%s)" % (start, nrows))
             step = 1
@@ -445,7 +539,8 @@ very small/large chunksize, you may want to increase/decrease it."""
                                                 warn_negstep=warn_negstep)
         return (start, stop, step)
 
-    def _g_copy(self, newparent, newname, recursive, _log=True, **kwargs):
+    def _g_copy(self, newparent: "Group", newname: str,
+                recursive: bool, _log: bool=True, **kwargs) -> "Leaf":
         # Compute default arguments.
         start = kwargs.pop('start', None)
         stop = kwargs.pop('stop', None)
@@ -486,8 +581,8 @@ very small/large chunksize, you may want to increase/decrease it."""
 
         return new_node
 
-    def _g_fix_byteorder_data(self, data, dbyteorder):
-        "Fix the byteorder of data passed in constructors."
+    def _g_fix_byteorder_data(self, data: np.ndarray, dbyteorder: str) -> np.ndarray:
+        """Fix the byteorder of data passed in constructors."""
         dbyteorder = byteorders[dbyteorder]
         # If self.byteorder has not been passed as an argument of
         # the constructor, then set it to the same value of data.
@@ -496,7 +591,7 @@ very small/large chunksize, you may want to increase/decrease it."""
         # Do an additional in-place byteswap of data if the in-memory
         # byteorder doesn't match that of the on-disk.  This is the only
         # place that we have to do the conversion manually. In all the
-        # other cases, it will be HDF5 the responsible of doing the
+        # other cases, it will be HDF5 the responsible for doing the
         # byteswap properly.
         if dbyteorder in ['little', 'big']:
             if dbyteorder != self.byteorder:
@@ -510,17 +605,17 @@ very small/large chunksize, you may want to increase/decrease it."""
             self.byteorder = "irrelevant"
         return data
 
-    def _point_selection(self, key):
+    def _point_selection(self, key: Union[list, tuple, np.ndarray]) -> np.ndarray:
         """Perform a point-wise selection.
 
         `key` can be any of the following items:
 
-        * A boolean array with the same shape than self. Those positions
+        * A boolean array with the same shape as self. Those positions
           with True values will signal the coordinates to be returned.
 
         * A numpy array (or list or tuple) with the point coordinates.
           This has to be a two-dimensional array of size len(self.shape)
-          by num_elements containing a list of of zero-based values
+          by num_elements containing a list of zero-based values
           specifying the coordinates in the dataset of the selected
           elements. The order of the element coordinates in the array
           specifies the order in which the array elements are iterated
@@ -534,30 +629,30 @@ very small/large chunksize, you may want to increase/decrease it."""
         point-wise selection.
 
         """
-
+        input_key = key
         if type(key) in (list, tuple):
             if isinstance(key, tuple) and len(key) > len(self.shape):
-                raise IndexError("Invalid index or slice: %r" % (key,))
+                raise IndexError(f"Invalid index or slice: {key!r}")
             # Try to convert key to a numpy array.  If not possible,
             # a TypeError will be issued (to be catched later on).
             try:
-                key = numpy.array(key)
+                key = toarray(key)
             except ValueError:
-                raise TypeError("Invalid index or slice: %r" % (key,))
-        elif not isinstance(key, numpy.ndarray):
-            raise TypeError("Invalid index or slice: %r" % (key,))
+                raise TypeError(f"Invalid index or slice: {key!r}")
+        elif not isinstance(key, np.ndarray):
+            raise TypeError(f"Invalid index or slice: {key!r}")
 
         # Protection against empty keys
         if len(key) == 0:
-            return numpy.array([], dtype="i8")
+            return np.array([], dtype="i8")
 
         if key.dtype.kind == 'b':
             if not key.shape == self.shape:
                 raise IndexError(
                     "Boolean indexing array has incompatible shape")
             # Get the True coordinates (64-bit indices!)
-            coords = numpy.asarray(key.nonzero(), dtype='i8')
-            coords = numpy.transpose(coords)
+            coords = np.asarray(key.nonzero(), dtype='i8')
+            coords = np.transpose(coords)
         elif key.dtype.kind == 'i' or key.dtype.kind == 'u':
             if len(key.shape) > 2:
                 raise IndexError(
@@ -566,18 +661,23 @@ very small/large chunksize, you may want to increase/decrease it."""
                 if key.shape[0] != len(self.shape):
                     raise IndexError(
                         "Coordinate indexing array has incompatible shape")
-                coords = numpy.asarray(key, dtype="i8")
-                coords = numpy.transpose(coords)
+                coords = np.asarray(key, dtype="i8")
+                coords = np.transpose(coords)
             else:
                 # For 1-dimensional datasets
-                coords = numpy.asarray(key, dtype="i8")
+                coords = np.asarray(key, dtype="i8")
 
             # handle negative indices
+            base = coords if coords.base is None else coords.base
+            if base is input_key:
+                # never modify the original "key" data
+                coords = coords.copy()
+
             idx = coords < 0
             coords[idx] = (coords + self.shape)[idx]
 
             # bounds check
-            if numpy.any(coords < 0) or numpy.any(coords >= self.shape):
+            if np.any(coords < 0) or np.any(coords >= self.shape):
                 raise IndexError("Index out of bounds")
         else:
             raise TypeError("Only integer coordinates allowed.")
@@ -586,11 +686,26 @@ very small/large chunksize, you may want to increase/decrease it."""
             coords = coords.copy()
         return coords
 
-    # Public methods
-    # ~~~~~~~~~~~~~~
+    def _check_chunked(self) -> None:
+        if self.chunkshape is None:
+            raise NotChunkedError("The dataset is not chunked")
+
+    def _check_chunk_within(self, coords: tuple[int, ...]) -> None:
+        if len(coords) != self.ndim:
+            raise ValueError(f"Chunk coordinates do not match dataset shape: "
+                             f"{coords} !~ {self.shape}")
+        if any(c < 0 or c >= s for (c, s) in zip(coords, self.shape)):
+            raise IndexError(f"Chunk coordinates not within dataset shape: "
+                             f"{coords} <> {self.shape}")
+
+    def _check_chunk_coords(self, coords: tuple[int, ...]) -> None:
+        if any(c % cs for (c, cs) in zip(coords, self.chunkshape)):
+            raise NotChunkAlignedError(
+                f"Coordinates are not multiples of chunk shape: "
+                f"{tuple(coords)} !* {self.chunkshape}")
+
     # Tree manipulation
-    # `````````````````
-    def remove(self):
+    def remove(self) -> None:
         """Remove this node from the hierarchy.
 
         This method has the behavior described
@@ -601,7 +716,7 @@ very small/large chunksize, you may want to increase/decrease it."""
 
         self._f_remove(False)
 
-    def rename(self, newname):
+    def rename(self, newname: str) -> None:
         """Rename this node in place.
 
         This method has the behavior described in :meth:`Node._f_rename()`.
@@ -610,8 +725,8 @@ very small/large chunksize, you may want to increase/decrease it."""
 
         self._f_rename(newname)
 
-    def move(self, newparent=None, newname=None,
-             overwrite=False, createparents=False):
+    def move(self, newparent: Optional["Group"]=None, newname: Optional[str]=None,
+             overwrite: bool=False, createparents: bool=False) -> None:
         """Move or rename this node.
 
         This method has the behavior described in :meth:`Node._f_move`
@@ -620,8 +735,8 @@ very small/large chunksize, you may want to increase/decrease it."""
 
         self._f_move(newparent, newname, overwrite, createparents)
 
-    def copy(self, newparent=None, newname=None,
-             overwrite=False, createparents=False, **kwargs):
+    def copy(self, newparent: Optional["Group"]=None, newname: Optional[str]=None,
+             overwrite: bool=False, createparents: bool=False, **kwargs) -> "Leaf":
         """Copy this node and return the new one.
 
         This method has the behavior described in :meth:`Node._f_copy`. Please
@@ -660,18 +775,18 @@ very small/large chunksize, you may want to increase/decrease it."""
         chunkshape
             The chunkshape of the new leaf.  It supports a couple of special
             values.  A value of keep means that the chunkshape will be the same
-            than original leaf (this is the default).  A value of auto means
+            as original leaf (this is the default).  A value of auto means
             that a new shape will be computed automatically in order to ensure
-            best performance when accessing the dataset through the main
+            the best performance when accessing the dataset through the main
             dimension.  Any other value should be an integer or a tuple
             matching the dimensions of the leaf.
 
         """
 
         return self._f_copy(
-            newparent, newname, overwrite, createparents, **kwargs)
+            newparent, newname, overwrite, createparents=createparents, **kwargs)
 
-    def truncate(self, size):
+    def truncate(self, size: int) -> None:
         """Truncate the main dimension to be size rows.
 
         If the main dimension previously was larger than this size, the extra
@@ -688,7 +803,7 @@ very small/large chunksize, you may want to increase/decrease it."""
             raise TypeError("non-enlargeable datasets cannot be truncated")
         self._g_truncate(size)
 
-    def isvisible(self):
+    def isvisible(self) -> bool:
         """Is this node visible?
 
         This method has the behavior described in :meth:`Node._f_isvisible()`.
@@ -698,8 +813,7 @@ very small/large chunksize, you may want to increase/decrease it."""
         return self._f_isvisible()
 
     # Attribute handling
-    # ``````````````````
-    def get_attr(self, name):
+    def get_attr(self, name: str) -> Any:
         """Get a PyTables attribute from this node.
 
         This method has the behavior described in :meth:`Node._f_getattr`.
@@ -708,7 +822,7 @@ very small/large chunksize, you may want to increase/decrease it."""
 
         return self._f_getattr(name)
 
-    def set_attr(self, name, value):
+    def set_attr(self, name: str, value: Any) -> None:
         """Set a PyTables attribute for this node.
 
         This method has the behavior described in :meth:`Node._f_setattr()`.
@@ -717,7 +831,7 @@ very small/large chunksize, you may want to increase/decrease it."""
 
         self._f_setattr(name, value)
 
-    def del_attr(self, name):
+    def del_attr(self, name: str) -> None:
         """Delete a PyTables attribute from this node.
 
         This method has the behavior described in :meth:`Node_f_delAttr`.
@@ -727,8 +841,7 @@ very small/large chunksize, you may want to increase/decrease it."""
         self._f_delattr(name)
 
     # Data handling
-    # `````````````
-    def flush(self):
+    def flush(self) -> None:
         """Flush pending data to disk.
 
         Saves whatever remaining buffered data to disk. It also releases
@@ -740,7 +853,118 @@ very small/large chunksize, you may want to increase/decrease it."""
 
         self._g_flush()
 
-    def _f_close(self, flush=True):
+    def chunk_info(self, coords: tuple[int, ...]) -> ChunkInfo:
+        """Get storage information about the chunk containing the `coords`.
+
+        The coordinates `coords` are a tuple of integers with the same rank as
+        the dataset.
+
+        Return a :class:`ChunkInfo` instance with the information.
+
+        The coordinates need not be aligined with chunk boundaries.  This
+        means that this method may be used to get the start coordinates of the
+        chunk that contains the item at the given coordinates, for use with
+        other direct chunking operations (see :attr:`ChunkInfo.start`).
+
+        If the coordinates are within the dataset's shape but there is no such
+        chunk in storage (missing chunk), a :class:`ChunkInfo` with a valid
+        ``start`` and ``filter_mask = offset = size = None`` is returned.  If
+        the coordinates are beyond the shape, :exc:`IndexError` is raised
+        (even if the start of the chunk would fall within the shape).
+
+        Calling this method on a non-chunked dataset raises a
+        :exc:`NotChunkedError`.
+
+        """
+        self._check_chunked()
+        self._check_chunk_within(coords)
+
+        coords = np.array(coords, dtype=SizeType)
+        filter_mask, offset, size = self._g_chunk_info(coords)
+
+        # Align coordinates to chunk boundary.
+        chunkshape = self.chunkshape
+        coords //= chunkshape
+        coords *= chunkshape
+        return ChunkInfo(tuple(coords), filter_mask, offset, size)
+
+    def read_chunk(self, coords: tuple[int, ...],
+                   out: Optional[Union[bytearray, NPByteArray]]=None,
+                  ) -> Union[bytes, memoryview]:
+        """Get the raw chunk that starts at the given `coords` from storage.
+
+        The coordinates `coords` are a tuple of integers with the same rank as
+        the dataset.  If they are not multiples of its chunkshape,
+        :exc:`NotChunkAlignedError` is raised.
+
+        If a buffer-like `out` argument is given, it receives chunk data.  If
+        it has insufficient storage for the chunk, :exc:`ValueError` is raised
+        (use :meth:`chunk_info()` to get the required capacity).
+
+        The obtained data is supposed to have gone at storage time through
+        dataset filters, minus those in the chunk's filter mask (use
+        :meth:`chunk_info()` to get it).
+
+        Return the chunk's raw content, either as a `bytes` instance (if `out`
+        is ``None``) or as a `memoryview` over the object given as `out`.
+
+        Reading a chunk within the dataset's shape, but not in storage
+        (missing chunk) raises a :exc:`NoSuchChunkError`.  If the chunk is
+        beyond the shape, :exc:`IndexError` is raised.
+
+        Calling this method on a non-chunked dataset raises a
+        :exc:`NotChunkedError`.
+
+        """
+        self._check_chunked()
+        self._check_chunk_within(coords)
+        self._check_chunk_coords(coords)
+
+        if out is not None:
+            out = np.ndarray((len(out),), dtype='u1', buffer=out)
+
+        coords = np.array(coords, dtype=SizeType)
+        chunk = self._g_read_chunk(coords, out)
+        if chunk is None:
+            raise NoSuchChunkError(f"Can't read missing chunk at coordinates "
+                                   f"{tuple(coords)}")
+        return chunk.tobytes() if out is None else memoryview(out)
+
+    def write_chunk(self, coords: tuple[int, ...], data: BufferLike,
+                    filter_mask: int=0) -> None:
+        """Write raw `data` to storage for the chunk that starts at the given
+        `coords`.
+
+        The coordinates `coords` are a tuple of integers with the same rank as
+        the dataset.  If they are not multiples of its chunkshape,
+        :exc:`NotChunkAlignedError` is raised.
+
+        The content of the buffer-like `data` must already have gone through
+        dataset filters, minus those in the given `filter_mask` (which is to
+        be saved along data; see :attr:`ChunkInfo.filter_mask`).
+
+        Writing a chunk which is already in storage replaces it, otherwise it
+        is added to storage as long as it is within the dataset's shape
+        (missing chunk).  This means that you may use :meth:`truncate()` to
+        grow an enlargeable dataset cheaply (as no chunk data is written),
+        then sparsely write selected chunks in arbitrary order.
+
+        If the chunk is beyond the dataset's shape, :exc:`IndexError` is
+        raised.
+
+        Calling this method on a non-chunked dataset raises a
+        :exc:`NotChunkedError`.
+
+        """
+        self._check_chunked()
+        self._check_chunk_within(coords)
+        self._check_chunk_coords(coords)
+
+        coords = np.array(coords, dtype=SizeType)
+        data = np.ndarray((len(data),), dtype='u1', buffer=data)
+        self._g_write_chunk(coords, data, filter_mask)
+
+    def _f_close(self, flush: bool=True) -> None:
         """Close this node in the tree.
 
         This method has the behavior described in :meth:`Node._f_close`.
@@ -763,9 +987,9 @@ very small/large chunksize, you may want to increase/decrease it."""
         self._g_close()
 
         # Close myself as a node.
-        super(Leaf, self)._f_close()
+        super()._f_close()
 
-    def close(self, flush=True):
+    def close(self, flush: bool=True) -> None:
         """Close this node in the tree.
 
         This method is completely equivalent to :meth:`Leaf._f_close`.
@@ -773,11 +997,3 @@ very small/large chunksize, you may want to increase/decrease it."""
         """
 
         self._f_close(flush)
-
-
-## Local Variables:
-## mode: python
-## py-indent-offset: 4
-## tab-width: 4
-## fill-column: 72
-## End:
