@@ -255,7 +255,7 @@ class OpenFileTestCase(common.TempFileMixin, common.PyTablesTestCase):
         self.assertEqual(title, "Array title 1")
 
     # This test works well, but HDF5 emits a series of messages that
-    # may loose the user. It is better to deactivate it.
+    # may lose the user. It is better to deactivate it.
     def notest04c_alternateRootFile(self):
         """Checking non-existent alternate root access to the object tree"""
 
@@ -1310,7 +1310,7 @@ class CheckFileTestCase(common.TempFileMixin, common.PyTablesTestCase):
 
     def test04c_UnImplementedScalar(self):
         """Checking opening of HDF5 files containing scalar dataset of
-        UnImlemented type."""
+        UnImplemented type."""
 
         with tb.open_file(common.test_filename("scalar.h5")) as h5file:
             with self.assertWarns(UserWarning):
@@ -2494,25 +2494,25 @@ def suite():
     niter = 1
 
     for i in range(niter):
-        theSuite.addTest(common.unittest.makeSuite(OpenFileFailureTestCase))
-        theSuite.addTest(common.unittest.makeSuite(NodeCacheOpenFile))
-        theSuite.addTest(common.unittest.makeSuite(NoNodeCacheOpenFile))
-        theSuite.addTest(common.unittest.makeSuite(DictNodeCacheOpenFile))
-        theSuite.addTest(common.unittest.makeSuite(CheckFileTestCase))
-        theSuite.addTest(common.unittest.makeSuite(ThreadingTestCase))
-        theSuite.addTest(common.unittest.makeSuite(PythonAttrsTestCase))
-        theSuite.addTest(common.unittest.makeSuite(StateTestCase))
-        theSuite.addTest(common.unittest.makeSuite(FlavorTestCase))
-        theSuite.addTest(common.unittest.makeSuite(UnicodeFilename))
-        theSuite.addTest(common.unittest.makeSuite(PathLikeFilename))
-        theSuite.addTest(common.unittest.makeSuite(FilePropertyTestCase))
-        theSuite.addTest(common.unittest.makeSuite(BloscBigEndian))
-        theSuite.addTest(common.unittest.makeSuite(BloscSubprocess))
-        theSuite.addTest(common.unittest.makeSuite(HDF5ErrorHandling))
-        theSuite.addTest(common.unittest.makeSuite(TestDescription))
-        theSuite.addTest(common.unittest.makeSuite(TestAtom))
-        theSuite.addTest(common.unittest.makeSuite(TestCol))
-        theSuite.addTest(common.unittest.makeSuite(TestSysattrCompatibility))
+        theSuite.addTest(common.make_suite(OpenFileFailureTestCase))
+        theSuite.addTest(common.make_suite(NodeCacheOpenFile))
+        theSuite.addTest(common.make_suite(NoNodeCacheOpenFile))
+        theSuite.addTest(common.make_suite(DictNodeCacheOpenFile))
+        theSuite.addTest(common.make_suite(CheckFileTestCase))
+        theSuite.addTest(common.make_suite(ThreadingTestCase))
+        theSuite.addTest(common.make_suite(PythonAttrsTestCase))
+        theSuite.addTest(common.make_suite(StateTestCase))
+        theSuite.addTest(common.make_suite(FlavorTestCase))
+        theSuite.addTest(common.make_suite(UnicodeFilename))
+        theSuite.addTest(common.make_suite(PathLikeFilename))
+        theSuite.addTest(common.make_suite(FilePropertyTestCase))
+        theSuite.addTest(common.make_suite(BloscBigEndian))
+        theSuite.addTest(common.make_suite(BloscSubprocess))
+        theSuite.addTest(common.make_suite(HDF5ErrorHandling))
+        theSuite.addTest(common.make_suite(TestDescription))
+        theSuite.addTest(common.make_suite(TestAtom))
+        theSuite.addTest(common.make_suite(TestCol))
+        theSuite.addTest(common.make_suite(TestSysattrCompatibility))
 
     return theSuite
 
