@@ -2515,68 +2515,68 @@ def suite():
     # common.heavy = 1 # Uncomment this only for testing purposes!
 
     for i in range(niter):
-        theSuite.addTest(common.unittest.makeSuite(FiltersCase1))
-        theSuite.addTest(common.unittest.makeSuite(FiltersCase2))
-        theSuite.addTest(common.unittest.makeSuite(FiltersCase10))
-        theSuite.addTest(common.unittest.makeSuite(FiltersCaseBloscBloscLZ))
-        theSuite.addTest(common.unittest.makeSuite(FiltersCaseBloscLZ4))
-        theSuite.addTest(common.unittest.makeSuite(FiltersCaseBloscLZ4HC))
-        theSuite.addTest(common.unittest.makeSuite(FiltersCaseBloscSnappy))
-        theSuite.addTest(common.unittest.makeSuite(FiltersCaseBloscZlib))
-        theSuite.addTest(common.unittest.makeSuite(FiltersCaseBloscZstd))
-        theSuite.addTest(common.unittest.makeSuite(FiltersCaseBloscBitShuffle))
-        theSuite.addTest(common.unittest.makeSuite(CopyGroupCase1))
-        theSuite.addTest(common.unittest.makeSuite(CopyGroupCase2))
-        theSuite.addTest(common.unittest.makeSuite(CopyFileCase1))
-        theSuite.addTest(common.unittest.makeSuite(CopyFileCase2))
-        theSuite.addTest(common.unittest.makeSuite(GroupFiltersTestCase))
-        theSuite.addTest(common.unittest.makeSuite(SetBloscMaxThreadsTestCase))
-        theSuite.addTest(common.unittest.makeSuite(FilterTestCase))
+        theSuite.addTest(common.make_suite(FiltersCase1))
+        theSuite.addTest(common.make_suite(FiltersCase2))
+        theSuite.addTest(common.make_suite(FiltersCase10))
+        theSuite.addTest(common.make_suite(FiltersCaseBloscBloscLZ))
+        theSuite.addTest(common.make_suite(FiltersCaseBloscLZ4))
+        theSuite.addTest(common.make_suite(FiltersCaseBloscLZ4HC))
+        theSuite.addTest(common.make_suite(FiltersCaseBloscSnappy))
+        theSuite.addTest(common.make_suite(FiltersCaseBloscZlib))
+        theSuite.addTest(common.make_suite(FiltersCaseBloscZstd))
+        theSuite.addTest(common.make_suite(FiltersCaseBloscBitShuffle))
+        theSuite.addTest(common.make_suite(CopyGroupCase1))
+        theSuite.addTest(common.make_suite(CopyGroupCase2))
+        theSuite.addTest(common.make_suite(CopyFileCase1))
+        theSuite.addTest(common.make_suite(CopyFileCase2))
+        theSuite.addTest(common.make_suite(GroupFiltersTestCase))
+        theSuite.addTest(common.make_suite(SetBloscMaxThreadsTestCase))
+        theSuite.addTest(common.make_suite(FilterTestCase))
         theSuite.addTest(doctest.DocTestSuite(tb.filters))
 
-        theSuite.addTest(common.unittest.makeSuite(DefaultDriverTestCase))
-        theSuite.addTest(common.unittest.makeSuite(Sec2DriverTestCase))
-        theSuite.addTest(common.unittest.makeSuite(StdioDriverTestCase))
-        theSuite.addTest(common.unittest.makeSuite(CoreDriverTestCase))
-        theSuite.addTest(common.unittest.makeSuite(
+        theSuite.addTest(common.make_suite(DefaultDriverTestCase))
+        theSuite.addTest(common.make_suite(Sec2DriverTestCase))
+        theSuite.addTest(common.make_suite(StdioDriverTestCase))
+        theSuite.addTest(common.make_suite(CoreDriverTestCase))
+        theSuite.addTest(common.make_suite(
             CoreDriverNoBackingStoreTestCase))
-        theSuite.addTest(common.unittest.makeSuite(SplitDriverTestCase))
+        theSuite.addTest(common.make_suite(SplitDriverTestCase))
 
-        theSuite.addTest(common.unittest.makeSuite(LogDriverTestCase))
-        theSuite.addTest(common.unittest.makeSuite(DirectDriverTestCase))
-        theSuite.addTest(common.unittest.makeSuite(WindowsDriverTestCase))
+        theSuite.addTest(common.make_suite(LogDriverTestCase))
+        theSuite.addTest(common.make_suite(DirectDriverTestCase))
+        theSuite.addTest(common.make_suite(WindowsDriverTestCase))
 
-        theSuite.addTest(common.unittest.makeSuite(FamilyDriverTestCase))
-        theSuite.addTest(common.unittest.makeSuite(MultiDriverTestCase))
-        theSuite.addTest(common.unittest.makeSuite(MpioDriverTestCase))
-        theSuite.addTest(common.unittest.makeSuite(MpiPosixDriverTestCase))
-        theSuite.addTest(common.unittest.makeSuite(StreamDriverTestCase))
-        theSuite.addTest(common.unittest.makeSuite(InMemoryCoreDriverTestCase))
+        theSuite.addTest(common.make_suite(FamilyDriverTestCase))
+        theSuite.addTest(common.make_suite(MultiDriverTestCase))
+        theSuite.addTest(common.make_suite(MpioDriverTestCase))
+        theSuite.addTest(common.make_suite(MpiPosixDriverTestCase))
+        theSuite.addTest(common.make_suite(StreamDriverTestCase))
+        theSuite.addTest(common.make_suite(InMemoryCoreDriverTestCase))
 
-        theSuite.addTest(common.unittest.makeSuite(QuantizeTestCase))
+        theSuite.addTest(common.make_suite(QuantizeTestCase))
 
     if common.heavy:
-        theSuite.addTest(common.unittest.makeSuite(CreateTestCase))
-        theSuite.addTest(common.unittest.makeSuite(FiltersCase3))
-        theSuite.addTest(common.unittest.makeSuite(FiltersCase4))
-        theSuite.addTest(common.unittest.makeSuite(FiltersCase5))
-        theSuite.addTest(common.unittest.makeSuite(FiltersCase6))
-        theSuite.addTest(common.unittest.makeSuite(FiltersCase7))
-        theSuite.addTest(common.unittest.makeSuite(FiltersCase8))
-        theSuite.addTest(common.unittest.makeSuite(FiltersCase9))
-        theSuite.addTest(common.unittest.makeSuite(CopyFileCase3))
-        theSuite.addTest(common.unittest.makeSuite(CopyFileCase4))
-        theSuite.addTest(common.unittest.makeSuite(CopyFileCase5))
-        theSuite.addTest(common.unittest.makeSuite(CopyFileCase6))
-        theSuite.addTest(common.unittest.makeSuite(CopyFileCase7))
-        theSuite.addTest(common.unittest.makeSuite(CopyFileCase8))
-        theSuite.addTest(common.unittest.makeSuite(CopyFileCase10))
-        theSuite.addTest(common.unittest.makeSuite(CopyGroupCase3))
-        theSuite.addTest(common.unittest.makeSuite(CopyGroupCase4))
-        theSuite.addTest(common.unittest.makeSuite(CopyGroupCase5))
-        theSuite.addTest(common.unittest.makeSuite(CopyGroupCase6))
-        theSuite.addTest(common.unittest.makeSuite(CopyGroupCase7))
-        theSuite.addTest(common.unittest.makeSuite(CopyGroupCase8))
+        theSuite.addTest(common.make_suite(CreateTestCase))
+        theSuite.addTest(common.make_suite(FiltersCase3))
+        theSuite.addTest(common.make_suite(FiltersCase4))
+        theSuite.addTest(common.make_suite(FiltersCase5))
+        theSuite.addTest(common.make_suite(FiltersCase6))
+        theSuite.addTest(common.make_suite(FiltersCase7))
+        theSuite.addTest(common.make_suite(FiltersCase8))
+        theSuite.addTest(common.make_suite(FiltersCase9))
+        theSuite.addTest(common.make_suite(CopyFileCase3))
+        theSuite.addTest(common.make_suite(CopyFileCase4))
+        theSuite.addTest(common.make_suite(CopyFileCase5))
+        theSuite.addTest(common.make_suite(CopyFileCase6))
+        theSuite.addTest(common.make_suite(CopyFileCase7))
+        theSuite.addTest(common.make_suite(CopyFileCase8))
+        theSuite.addTest(common.make_suite(CopyFileCase10))
+        theSuite.addTest(common.make_suite(CopyGroupCase3))
+        theSuite.addTest(common.make_suite(CopyGroupCase4))
+        theSuite.addTest(common.make_suite(CopyGroupCase5))
+        theSuite.addTest(common.make_suite(CopyGroupCase6))
+        theSuite.addTest(common.make_suite(CopyGroupCase7))
+        theSuite.addTest(common.make_suite(CopyGroupCase8))
 
     return theSuite
 
